@@ -101,6 +101,7 @@ parser.add_argument("--use-ini",default=None,type=str,help="Pass ini file for pa
 parser.add_argument("--use-rundir",default=None,type=str,help="Intended to reproduce lalinference_pipe functionality. Must be absolute path.")
 parser.add_argument("--use-online-psd-file",default=None,type=str,help="Provides specific online PSD file, so no downloads are needed")
 parser.add_argument("--use-coinc",default=None,type=str,help="Intended to reproduce lalinference_pipe functionality")
+parser.add_argument("--manual-ifo-list",default=None,type=str,help="Overrides IFO list normally retrieve by event ID.  Use with care (e.g., glitch studies) or for events specified with --event-time.")
 parser.add_argument("--online",action='store_true')
 parser.add_argument("--extra-args-helper",action=None, help="Filename with arguments for the helper. Use to provide alternative channel names and other advanced configuration (--channel-name, data type)!")
 parser.add_argument("--manual-postfix",default='',type=str)
@@ -112,6 +113,8 @@ parser.add_argument("--approx",default=None,type=str,help="Approximant. REQUIRED
 parser.add_argument("--use-gwsurrogate",action='store_true',help="Attempt to use gwsurrogate instead of lalsuite.")
 parser.add_argument("--l-max",default=2,type=int)
 parser.add_argument("--no-matter",action='store_true', help="Force analysis without matter. Really only matters for BNS")
+parser.add_argument("--assume-precessing",action='store_true', help="Force analysis *with* transverse spins")
+parser.add_argument("--assume-nonprecessing",action='store_true', help="Force analysis *without* transverse spins")
 parser.add_argument("--assume-matter",action='store_true', help="Force analysis *with* matter. Really only matters for BNS")
 parser.add_argument("--assume-highq",action='store_true', help="Force analysis with the high-q strategy, neglecting spin2. Passed to 'helper'")
 parser.add_argument("--assume-well-placed",action='store_true',help="If present, the code will adopt a strategy that assumes the initial grid is very well placed, and will minimize the number of early iterations performed. Not as extrme as --propose-flat-strategy")
@@ -127,8 +130,11 @@ parser.add_argument("--cip-sampler-method",type=str,default=None)
 parser.add_argument("--cip-fit-method",type=str,default=None)
 parser.add_argument("--ile-jobs-per-worker",type=int,default=20)
 parser.add_argument("--ile-no-gpu",action='store_true')
+parser.add_argument("--ile-force-gpu",action='store_true')
 parser.add_argument("--spin-magnitude-prior",default='default',type=str,help="options are default [volumetric for precessing,uniform for aligned], volumetric, uniform_mag_prec, uniform_mag_aligned, zprior_aligned")
 parser.add_argument("--force-chi-max",default=None,type=float,help="Provde this value to override the value of chi-max provided") 
+parser.add_argument("--force-mc-range",default=None,type=str,help="Pass this argumen through to the helper to set the mc range")
+parser.add_argument("--force-eta-range",default=None,type=str,help="Pass this argumen through to the helper to set the eta range")
 parser.add_argument("--hierarchical-merger-prior-1g",action='store_true',help="As in 1903.06742")
 parser.add_argument("--hierarchical-merger-prior-2g",action='store_true',help="As in 1903.06742")
 parser.add_argument("--link-reference-pe",action='store_true',help="If present, creates a directory 'reference_pe' and adds symbolic links to fiducial samples. These can be used by the automated plotting code.  Requires LVC_PE_SAMPLES environment variable defined!")
@@ -238,8 +244,13 @@ if opts.choose_data_LI_seglen:
 
 
 is_analysis_precessing =False
-if opts.approx == "SEOBNRv3" or opts.approx == "NRSur7dq2" or opts.approx == "NRSur7dq4" or (opts.approx == 'SEOBNv3_opt') or (opts.approx == 'IMRPhenomPv2') or (opts.approx =="SEOBNRv4P" ) or (opts.approx == "SEOBNRv4PHM") or ('SpinTaylor' in opts.approx):
+if opts.approx == "SEOBNRv3" or opts.approx == "NRSur7dq2" or opts.approx == "NRSur7dq4" or (opts.approx == 'SEOBNv3_opt') or (opts.approx == 'IMRPhenomPv2') or (opts.approx =="SEOBNRv4P" ) or (opts.approx == "SEOBNRv4PHM") or ('SpinTaylor' in opts.approx) or ('IMRPhenomTP' in opts.approx or ('IMRPhenomXP' in opts.approx)):
         is_analysis_precessing=True
+if opts.assume_precessing:
+        is_analysis_precessing = True
+if opts.assume_nonprecessing:
+        is_analysis_precessing = False
+
 
 dirname_run = gwid+ "_" + opts.calibration+ "_"+ opts.approx+"_fmin" + str(fmin) +"_fmin-template"+str(fmin_template) +"_lmax"+str(opts.l_max) + "_"+opts.spin_magnitude_prior
 if opts.online:
@@ -303,7 +314,6 @@ if not(opts.use_ini is None):
     lalsimutils.ChooseWaveformParams_array_to_xml([P], "target_params")
 
 
-
 helper_psd_args = ''
 srate=4096  # default, built into helper, unwise to go lower, LI will almost never do higher
 if opts.make_bw_psds:
@@ -332,9 +342,14 @@ if True:
                     except:
                         print( " No provenance for ", name)
 
+
 # Run helper command
 npts_it = 500
 cmd = " helper_LDG_Events.py --force-notune-initial-grid   --propose-fit-strategy --propose-ile-convergence-options --propose-initial-grid --fmin " + str(fmin) + " --fmin-template " + str(fmin_template) + " --working-directory " + base_dir + "/" + dirname_run  + helper_psd_args  + " --no-enforce-duration-bound "
+if not(opts.force_mc_range is None):
+    cmd+= " --force-mc-range {} ".format(opts.force_mc_range)
+if not(opts.force_eta_range is None):
+    cmd+= " --force-eta-range {} ".format(opts.force_eta_range)
 if not(opts.gracedb_id is None) and (opts.use_ini is None):
     cmd +=" --use-legacy-gracedb --gracedb-id " + gwid 
 elif  not(opts.event_time is None):
@@ -368,7 +383,7 @@ if opts.use_osg:
     cmd += " --use-cvmfs-frames "  # only run with CVMFS data, otherwise very very painful
 if opts.use_ini:
     cmd += " --use-ini " + opts.use_ini
-    cmd += " --sim-xml target_params.xml.gz --event 0 "
+    cmd += " --sim-xml {}/target_params.xml.gz --event 0 ".format(base_dir + "/"+ dirname_run)  # full path to target_params.xml.gz
     cmd += " --event-time " + str(event_dict["tref"])
     #
 else:
@@ -384,7 +399,21 @@ if opts.use_online_psd_file:
         cmd+= " --psd-file {}={}".format(ifo,opts.use_online_psd_file)
 if "SNR" in event_dict:
     cmd += " --hint-snr {} ".format(event_dict["SNR"])
+if not(opts.event_time is None) and not(opts.manual_ifo_list is None):
+    cmd += " --manual-ifo-list {} ".format(opts.manual_ifo_list)
 
+
+# If user provides ini file *and* ini file has fake-cache field, generate a local.cache file, and pass it as argument
+if opts.use_ini:
+    config = ConfigParser.ConfigParser()
+    config.read(opts.use_ini)
+    if config.has_option("lalinference", "fake-cache"):
+        # dictionary, entries are individual lcf files; we just need to concatenate their contents
+        fake_cache_dict = unsafe_config_get("lalinference","fake-cache")
+        fake_cache_fnames = [fake_cache_dict[x] for x in fake_cache_dict.keys()]
+        cmd_cat = 'cat ' + ' '.join(fake_cache_fnames) + ' > local.cache'
+        os.system(cmd_cat)
+        cmd += " --cache local.cache "
 print( cmd)
 os.system(cmd)
 #sys.exit(0)
@@ -421,6 +450,8 @@ line = ' '.join(instructions_ile)
 line += " --l-max " + str(opts.l_max) 
 if (opts.use_ini is None):
     line += " --d-max " + str(dmax_guess)
+if opts.ile_force_gpu:
+    line +=" --force-gpu-only "
 sur_location_prefix = "my_surrogates/nr_surrogates/"
 if opts.use_osg:
     sur_location_prefix = "/"
@@ -462,7 +493,11 @@ for indx in np.arange(len(instructions_cip)):
     n_max_cip = 10000000;
     if opts.cip_sampler_method is "GMM":
         n_max_cip *=10   # it is faster, so run longer; helps with correlated-sampling cases
-    line +=" --n-output-samples 10000 --n-eff 10000 --n-max {}   --downselect-parameter m2 --downselect-parameter-range [1,1000] ".format(n_max_cip)
+    n_sample_target=10000
+    n_workers = 1
+    if opts.cip_explode_jobs:
+        n_workers = opts.cip_explode_jobs
+    line +=" --n-output-samples {}  --n-eff {} --n-max {}   --downselect-parameter m2 --downselect-parameter-range [1,1000] ".format(int(n_sample_target/n_workers), int(n_sample_target/n_workers),n_max_cip)
     if not(opts.cip_fit_method is None):
         line = line.replace('--fit-method gp', '--fit-method ' + opts.cip_fit_method)
     if not (opts.cip_sampler_method is None):
@@ -603,3 +638,19 @@ if opts.archive_pesummary_label:
     cmd += " --plot-exe summarypages --plot-args  args_plot.txt "
 print(cmd)
 os.system(cmd)
+
+
+
+## RUNMON
+try:
+    from runmonitor import store_tools as sto
+    if opts.use_ini != None: # making an assumption that opts.use_ini corresponds to prod_O3b file structures, and that opts.use_ini == None corresponds to standard setup with opts.gracedb_id passed. Maybe not a robust assumptio
+        level = 2
+        event = os.getcwd.split("/")[-2].split("_")[0]
+    else:
+        level = 1
+        event = opts.gracedb_id
+    sto.store(event,level)
+except Exception as fail:
+    print(fail)
+    print("Unable to initialize run monitoring automatically. If you wish to use this feature please do so manually")
